@@ -3,8 +3,13 @@ import os
 import queue
 import subprocess
 
+from typing import Any, Callable, Dict, Mapping, Optional, Union, overload
+
 from wrapt import synchronized
 
+from koapy.backend.kiwoom_open_api_plus.core.KiwoomOpenApiPlusDispatchFunctions import (
+    KiwoomOpenApiPlusDispatchFunctions,
+)
 from koapy.backend.kiwoom_open_api_plus.core.KiwoomOpenApiPlusError import (
     KiwoomOpenApiPlusError,
 )
@@ -13,7 +18,7 @@ from koapy.backend.kiwoom_open_api_plus.core.KiwoomOpenApiPlusRateLimiter import
     KiwoomOpenApiPlusSendConditionRateLimiter,
     KiwoomOpenApiPlusSendOrderRateLimiter,
 )
-from koapy.backend.kiwoom_open_api_plus.utils.list_type import string_to_list
+from koapy.backend.kiwoom_open_api_plus.utils.list_conversion import string_to_list
 from koapy.config import config
 from koapy.utils.ctypes import is_admin
 from koapy.utils.logging.Logging import Logging
@@ -21,23 +26,96 @@ from koapy.utils.rate_limiting.pyside2.QRateLimitedExecutor import QRateLimitedE
 from koapy.utils.subprocess import function_to_subprocess_args
 
 
-class KiwoomOpenApiPlusSimpleQAxWidgetMixin:
+class KiwoomOpenApiPlusQAxWidgetUniversalMixin(KiwoomOpenApiPlusDispatchFunctions):
     def IsConnected(self):
         return self.GetConnectState() == 1
 
-    def GetServerGubun(self):
-        return self.GetLoginInfo("GetServerGubun")
-
-    def IsSimulationServer(self):
-        return self.GetServerGubun() == "1"
-
-    def IsRealServer(self):
-        return not self.IsSimulationServer()
-
     def ShowAccountWindow(self):
         return self.KOA_Functions("ShowAccountWindow", "")
 
-    def GetCodeListByMarketAsList(self, market=None):
+    def GetServerGubun(self):
+        gubun = self.KOA_Functions("GetServerGubun", "")
+        if not gubun:
+            gubun = self.GetLoginInfo("GetServerGubun")
+        return gubun
+
+    def IsSimulationServer(self):
+        gubun = self.GetServerGubun()
+        return gubun == "1"
+
+    def IsRealServer(self):
+        return not self.IsSimulationServer()
+
+    def GetMasterStockInfo(self, code):
+        return self.KOA_Functions("GetMasterStockInfo", code)
+
+    def GetMasterStockInfoAsDict(self, code):
+        result = self.GetMasterStockInfo(code)
+        items = string_to_list(result, sep=";")
+        items = [string_to_list(item, sep="|") for item in items]
+        info = dict(items)
+        return info
+
+    def SetConditionSearchFlag(self, flag):
+        return self.KOA_Functions("SetConditionSearchFlag", flag)
+
+    def AddPriceToConditionSearchResult(self):
+        return self.SetConditionSearchFlag("AddPrice")
+
+    def DelPriceFromConditionSearchResult(self):
+        return self.SetConditionSearchFlag("DelPrice")
+
+    def GetUpjongCode(self, code):
+        """
+        두번째 인자로 사용할 수 있는 값은 0, 1, 2, 4, 7 입니다.
+        0:코스피, 1: 코스닥, 2:KOSPI200, 4:KOSPI100(KOSPI50), 7:KRX100
+        """
+        code = str(code)
+        return self.KOA_Functions("GetUpjongCode", code)
+
+    def GetUpjongCodeAsList(self, code):
+        result = self.GetUpjongCode(code)
+        items = string_to_list(result, sep="|")
+        items = [string_to_list(item, sep=",") for item in items]
+        items = [tuple(item) for item in items]
+        return items
+
+    def GetUpjongNameByCode(self, code):
+        return self.KOA_Functions("GetUpjongNameByCode", code)
+
+    def IsOrderWarningETF(self, code):
+        """
+        투자유의 종목인 경우 "1" 값이 리턴, 그렇지 않은 경우 "0" 값 리턴. (ETF가 아닌 종목을 입력시 "0" 값 리턴.)
+        """
+        return self.KOA_Functions("IsOrderWarningETF", code)
+
+    def IsOrderWarningETFAsBoolean(self, code):
+        return_code = self.IsOrderWarningETF(code)
+        return_code = int(return_code)
+        return_code = bool(return_code)
+        return return_code
+
+    def IsOrderWarningStock(self, code):
+        """
+        리턴 값 - "0":해당없음, "2":정리매매, "3":단기과열, "4":투자위험, "5":투자경고
+        """
+        return self.KOA_Functions("IsOrderWarningStock", code)
+
+    def IsOrderWarningStockAsBoolean(self, code):
+        return_code = self.IsOrderWarningStock(code)
+        return_code = int(return_code)
+        return_code = bool(return_code)
+        return return_code
+
+    def GetMasterListedStockCntEx(self, code):
+        return self.KOA_Functions("GetMasterListedStockCntEx", code)
+
+    def GetMasterListedStockCntExAsInt(self, code):
+        count = self.GetMasterListedStockCntEx(code)
+        count = int(count)
+        return count
+
+    def GetCodeListByMarketAsList(self, market: Optional[Union[str, int]] = None):
         if market is None:
             market = ""
         market = str(market)
@@ -45,7 +123,7 @@ class KiwoomOpenApiPlusSimpleQAxWidgetMixin:
         result = string_to_list(result)
         return result
 
-    def GetNameListByMarketAsList(self, market):
+    def GetNameListByMarketAsList(self, market: Optional[Union[str, int]] = None):
         codes = self.GetCodeListByMarketAsList(market)
         names = [self.GetMasterCodeName(code) for code in codes]
         return names
@@ -89,7 +167,7 @@ class KiwoomOpenApiPlusSimpleQAxWidgetMixin:
             account = accounts[0]
         return account
 
-    def GetMasterStockStateAsList(self, code):
+    def GetMasterStockStateAsList(self, code: str):
         states = self.GetMasterStockState(code).strip()
         states = string_to_list(states, sep="|")
         return states
@@ -106,12 +184,12 @@ class KiwoomOpenApiPlusSimpleQAxWidgetMixin:
 
     def GetGeneralCodeList(
         self,
-        include_preferred_stock=False,
-        include_etn=False,
-        include_etf=False,
-        include_mutual_fund=False,
-        include_reits=False,
-        include_kosdaq=False,
+        include_preferred_stock: bool = False,
+        include_etn: bool = False,
+        include_etf: bool = False,
+        include_mutual_fund: bool = False,
+        include_reits: bool = False,
+        include_kosdaq: bool = False,
     ):
         """
         [시장구분값]
@@ -161,29 +239,29 @@ class KiwoomOpenApiPlusSimpleQAxWidgetMixin:
 
         return codes
 
-    def GetStockStates(self, code):
+    def GetStockStates(self, code: str):
         return self.GetMasterStockStateAsList(code)
 
-    def GetSurveillanceFlag(self, code):
+    def GetSurveillanceFlag(self, code: str):
         return self.GetMasterConstruction(code)
 
-    def IsSuspended(self, code):
+    def IsSuspended(self, code: str):
         return "거래정지" in self.GetMasterStockStateAsList(code)
 
-    def IsUnderSurveillance(self, code):
+    def IsUnderSurveillance(self, code: str):
         return "감리종목" in self.GetMasterStockStateAsList(code)
 
-    def IsUnderAdministration(self, code):
+    def IsUnderAdministration(self, code: str):
         return "관리종목" in self.GetMasterStockStateAsList(code)
 
-    def IsFlaggedForCaution(self, code):
+    def IsFlaggedForCaution(self, code: str):
         flag = self.GetSurveillanceFlag(code)
         states = self.GetMasterStockStateAsList(code)
         flag_is_not_normal = flag != "정상"
         has_caution_state = "투자유의종목" in states
         return flag_is_not_normal or has_caution_state
 
-    def IsProblematic(self, code):
+    def IsNotNormal(self, code: str):
         flag = self.GetSurveillanceFlag(code)
         states = self.GetMasterStockStateAsList(code)
         flag_is_not_normal = flag != "정상"
@@ -191,8 +269,8 @@ class KiwoomOpenApiPlusSimpleQAxWidgetMixin:
         has_any_bad_state = any(state in states for state in bad_states)
         return flag_is_not_normal or has_any_bad_state
 
-    def IsNormal(self, code):
-        return not self.IsProblematic(code)
+    def IsNormal(self, code: str):
+        return not self.IsNotNormal(code)
 
     def GetConditionFilePath(self):
         module_path = self.GetAPIModulePath()
@@ -227,21 +305,21 @@ class KiwoomOpenApiPlusSimpleQAxWidgetMixin:
     # ======================================================================
 
     @classmethod
-    def LoginUsingPywinauto_Impl(cls, credential=None):
+    def LoginUsingPywinauto_Impl(cls, credentials: Optional[Mapping[str, Any]] = None):
         import pywinauto
 
-        if credential is None:
-            credential = config.get("koapy.backend.kiwoom_open_api_plus.credential")
+        if credentials is None:
+            credentials = config.get("koapy.backend.kiwoom_open_api_plus.credentials")
 
         is_in_development = False
         use_set_text = False
 
-        userid = credential.get("user_id")
-        password = credential.get("user_password")
-        cert = credential.get("cert_password")
+        userid = credentials.get("user_id")
+        password = credentials.get("user_password")
+        cert = credentials.get("cert_password")
 
         is_save_userid = True
-        is_simulation = credential.get("is_simulation")
+        is_simulation = credentials.get("is_simulation")
 
         desktop = pywinauto.Desktop(allow_magic_lookup=False)
         login_window = desktop.window(title="Open API Login")
@@ -275,7 +353,7 @@ class KiwoomOpenApiPlusSimpleQAxWidgetMixin:
                     pywinauto.keyboard.send_keys(password)
                     pywinauto.keyboard.send_keys("{TAB}")
             else:
-                raise RuntimeError("'user_password' not set, please check credential")
+                raise RuntimeError("'user_password' not set, please check credentials")
 
             if is_save_userid:
                 cls.logger.info("Checking to save userid")
@@ -298,7 +376,7 @@ class KiwoomOpenApiPlusSimpleQAxWidgetMixin:
                         pywinauto.keyboard.send_keys("{TAB}")
                 else:
                     raise RuntimeError(
-                        "'cert_password' not set, please check credential"
+                        "'cert_password' not set, please check credentials"
                     )
             else:
                 if login_window["Edit3"].is_enabled():
@@ -310,7 +388,11 @@ class KiwoomOpenApiPlusSimpleQAxWidgetMixin:
 
     @classmethod
     def LoginUsingPywinauto_RunScriptInSubprocess(
-        cls, credential=None, wait=False, timeout=None, check=False
+        cls,
+        credentials: Optional[Mapping[str, Any]] = None,
+        wait: bool = False,
+        timeout: bool = None,
+        check: bool = False,
     ):
         def main():
             # pylint: disable=redefined-outer-name,reimported,import-self
@@ -321,12 +403,12 @@ class KiwoomOpenApiPlusSimpleQAxWidgetMixin:
                 KiwoomOpenApiPlusQAxWidgetMixin,
             )
 
-            credential = json.load(sys.stdin)
-            KiwoomOpenApiPlusQAxWidgetMixin.LoginUsingPywinauto_Impl(credential)
+            credentials = json.load(sys.stdin)
+            KiwoomOpenApiPlusQAxWidgetMixin.LoginUsingPywinauto_Impl(credentials)
 
         args = function_to_subprocess_args(main)
         process = subprocess.Popen(args, stdin=subprocess.PIPE, text=True)
-        json.dump(credential, process.stdin)
+        json.dump(credentials, process.stdin)
 
         if wait:
             try:
@@ -348,30 +430,63 @@ class KiwoomOpenApiPlusSimpleQAxWidgetMixin:
 
         return process
 
-    def LoginUsingPywinauto(self, credential=None, wait=True, timeout=None, check=True):
+    def LoginUsingPywinauto(
+        self,
+        credentials: Optional[Mapping[str, Any]] = None,
+        wait: bool = True,
+        timeout: bool = None,
+        check: bool = True,
+    ):
         assert is_admin(), "Using pywinauto requires administrator permission"
         return self.LoginUsingPywinauto_RunScriptInSubprocess(
-            credential, wait=wait, timeout=timeout, check=check
+            credentials, wait=wait, timeout=timeout, check=check
         )
 
-    def CommConnectAndThen(self, credential=None, callback=None):
+    @overload
+    def CommConnectAndThen(
+        self,
+        credentials: Mapping[str, Any],
+        callback: Callable[[int], Any],
+    ) -> int:
+        ...
+
+    @overload
+    def CommConnectAndThen(self, credentials: Mapping[str, Any]) -> int:
+        ...
+
+    @overload
+    def CommConnectAndThen(self, callback: Callable[[int], Any]) -> int:
+        ...
+
+    @overload
+    def CommConnectAndThen(self) -> int:
+        ...
+
+    def CommConnectAndThen(
+        self,
+        credentials_or_callback=None,
+        callback_or_none=None,
+    ) -> int:
+        credentials = credentials_or_callback
+        callback = callback_or_none
+
         if (
             callback is None
-            and credential is not None
-            and not isinstance(credential, dict)
-            and callable(credential)
+            and credentials is not None
+            and not isinstance(credentials, dict)
+            and callable(credentials)
         ):
-            callback = credential
-            credential = None
+            callback = credentials
+            credentials = None
 
-        if credential is not None:
+        if credentials is not None:
             assert (
                 is_admin()
-            ), "CommConnectAndThen() method requires to be run as administrator if credential is given explicitly"
+            ), "CommConnectAndThen() method requires to be run as administrator if credentials is given explicitly"
             self.DisableAutoLogin()
         elif not self.IsAutoLoginEnabled() and is_admin():
-            credential = config.get(
-                "koapy.backend.kiwoom_open_api_plus.credential", None
+            credentials = config.get(
+                "koapy.backend.kiwoom_open_api_plus.credentials", None
             )
 
         def OnEventConnect(errcode):
@@ -382,31 +497,56 @@ class KiwoomOpenApiPlusSimpleQAxWidgetMixin:
         self.OnEventConnect.connect(OnEventConnect)
         errcode = KiwoomOpenApiPlusError.try_or_raise(self.CommConnect())
 
-        if credential is not None:
-            self.LoginUsingPywinauto(credential, wait=False)
+        if credentials is not None:
+            self.LoginUsingPywinauto(credentials, wait=False)
 
         return errcode
 
-    def Connect(self, credential=None):
+    def Connect(self, credentials: Optional[Mapping[str, Any]] = None) -> int:
         q = queue.Queue()
 
         def OnEventConnect(errcode):
             q.put(errcode)
 
-        self.CommConnectAndThen(credential, OnEventConnect)
+        self.CommConnectAndThen(credentials, OnEventConnect)
         errcode = KiwoomOpenApiPlusError.try_or_raise(q.get())
 
         return errcode
 
-    def EnsureConnectedAndThen(self, credential=None, callback=None):
+    @overload
+    def EnsureConnectedAndThen(
+        self,
+        credentials: Mapping[str, Any],
+        callback: Callable[[int], Any],
+    ) -> bool:
+        ...
+
+    @overload
+    def EnsureConnectedAndThen(self, credentials: Mapping[str, Any]) -> bool:
+        ...
+
+    @overload
+    def EnsureConnectedAndThen(self, callback: Callable[[int], Any]) -> bool:
+        ...
+
+    @overload
+    def EnsureConnectedAndThen(self) -> bool:
+        ...
+
+    def EnsureConnectedAndThen(
+        self, credentials_or_callback=None, callback_or_none=None
+    ) -> bool:
+        credentials = credentials_or_callback
+        callback = callback_or_none
+
         if (
             callback is None
-            and credential is not None
-            and not isinstance(credential, dict)
-            and callable(credential)
+            and credentials is not None
+            and not isinstance(credentials, dict)
+            and callable(credentials)
         ):
-            callback = credential
-            credential = None
+            callback = credentials
+            credentials = None
         is_connected = self.IsConnected()
         if not is_connected:
 
@@ -415,22 +555,24 @@ class KiwoomOpenApiPlusSimpleQAxWidgetMixin:
                     if callable(callback):
                         callback()
 
-            self.CommConnectAndThen(credential, OnEventConnect)
+            self.CommConnectAndThen(credentials, OnEventConnect)
         else:
             if callable(callback):
                 callback()
         return is_connected
 
-    def EnsureConnected(self, credential=None):
+    def EnsureConnected(self, credentials: Optional[Mapping[str, Any]] = None) -> bool:
         is_connected = self.IsConnected()
         if not is_connected:
-            self.Connect(credential)
+            self.Connect(credentials)
             is_connected = self.IsConnected()
             assert is_connected, "Could not ensure connected"
         return is_connected
 
 
-class KiwoomOpenApiPlusComplexQAxWidgetMixin(Logging):
+class KiwoomOpenApiPlusQAxWidgetServerSideMixin(
+    KiwoomOpenApiPlusDispatchFunctions, Logging
+):
     def __init__(self):
 
         """
@@ -492,7 +634,7 @@ class KiwoomOpenApiPlusComplexQAxWidgetMixin(Logging):
         self.destroyed.connect(self._cond_rate_limited_executor.shutdown)
         self.destroyed.connect(self._order_rate_limited_executor.shutdown)
 
-    def LoadCondition(self):
+    def LoadCondition(self) -> int:
         q = queue.Queue()
 
         def OnReceiveConditionVer(ret, msg):
@@ -518,7 +660,7 @@ class KiwoomOpenApiPlusComplexQAxWidgetMixin(Logging):
             self.OnReceiveConditionVer.disconnect(OnReceiveConditionVer)
         return return_code
 
-    def IsConditionLoaded(self):
+    def IsConditionLoaded(self) -> bool:
         # the original implementation of this function was like the following:
         #   condition_filepath = self.GetConditionFilePath()
         #   return os.path.exists(condition_filepath)
@@ -528,7 +670,7 @@ class KiwoomOpenApiPlusComplexQAxWidgetMixin(Logging):
         # so here we are using entrypoint-wide member variable to remember once the condition is loaded
         return self._is_condition_loaded
 
-    def EnsureConditionLoaded(self, force=False):
+    def EnsureConditionLoaded(self, force: bool = False) -> int:
         return_code = 0
         is_condition_loaded = self.IsConditionLoaded()
         if not is_condition_loaded or force:
@@ -538,7 +680,14 @@ class KiwoomOpenApiPlusComplexQAxWidgetMixin(Logging):
         assert return_code == 1, "Could not ensure condition loaded"
         return return_code
 
-    def CommRqDataWithInputs(self, rqname, trcode, prevnext, scrnno, inputs=None):
+    def CommRqDataWithInputs(
+        self,
+        rqname: str,
+        trcode: str,
+        prevnext: Union[str, int],
+        scrnno: str,
+        inputs: Optional[Dict[str, str]] = None,
+    ) -> int:
         if inputs:
             for k, v in inputs.items():
                 self.SetInputValue(k, v)
@@ -547,12 +696,19 @@ class KiwoomOpenApiPlusComplexQAxWidgetMixin(Logging):
         return code
 
     @synchronized
-    def AtomicCommRqData(self, rqname, trcode, prevnext, scrnno, inputs=None):
+    def AtomicCommRqData(
+        self,
+        rqname: str,
+        trcode: str,
+        prevnext: Union[str, int],
+        scrnno: str,
+        inputs: Optional[Dict[str, str]] = None,
+    ) -> int:
         return self.CommRqDataWithInputs(rqname, trcode, prevnext, scrnno, inputs)
 
 
 class KiwoomOpenApiPlusQAxWidgetMixin(
-    KiwoomOpenApiPlusSimpleQAxWidgetMixin, KiwoomOpenApiPlusComplexQAxWidgetMixin
+    KiwoomOpenApiPlusQAxWidgetUniversalMixin, KiwoomOpenApiPlusQAxWidgetServerSideMixin
 ):
 
     pass
