@@ -211,10 +211,17 @@ class API:
             quote_type,
             original_order_no,
         )
-        _msg = next(responses)
-        _tr = next(responses)
-        accept = next(responses)
-        accept_data = dict(zip(accept.single_data.names, accept.single_data.values))
+
+        MAX_ORDER_RES_CNT = 5
+        for i in range(MAX_ORDER_RES_CNT):
+            res = next(responses)
+            if (
+                res.name == "OnReceiveChejanData"
+                and res.arguments[0].string_value == "0"  # 주문접수 or 체결
+            ):
+                break
+
+        accept_data = dict(zip(res.single_data.names, res.single_data.values))
         result = {"orderOpened": {"id": accept_data["주문번호"]}}
         return result
 
@@ -465,9 +472,7 @@ class KiwoomOpenApiPlusStore(Logging, metaclass=MetaKiwoomOpenApiPlusStore):
         streamer = KiwoomOpenApiPlusEventStreamer(self.api, q)
         streamer.events()
 
-    def candles(
-        self, dataname, dtbegin, dtend, timeframe, compression
-    ):  # pylint: disable=unused-argument
+    def candles(self, dataname, dtbegin, dtend, timeframe, compression):
         kwargs = locals().copy()
         kwargs.pop("self")
         kwargs["q"] = q = queue.Queue()
@@ -645,7 +650,6 @@ class KiwoomOpenApiPlusStore(Logging, metaclass=MetaKiwoomOpenApiPlusStore):
                 )
                 self.put_notification(e)
                 self.broker._reject(oref)
-                return
 
             # Ids are delivered in different fields and all must be fetched to
             # match them (as executions) to the order generated here
@@ -761,7 +765,6 @@ class KiwoomOpenApiPlusStore(Logging, metaclass=MetaKiwoomOpenApiPlusStore):
             return
 
         try:
-            oref = self._ordersrev[oid]
             self._process_transaction(oid, trans)
         except KeyError:  # not yet seen, keep as pending
             self._transpend[oid].append(trans)
@@ -776,7 +779,7 @@ class KiwoomOpenApiPlusStore(Logging, metaclass=MetaKiwoomOpenApiPlusStore):
 
     def _process_transaction(self, oid, trans):
         try:
-            oref = self._ordersrev.pop(oid)
+            oref = self._ordersrev[oid]
         except KeyError:
             return
 
